UNIT GRAPH2D;

INTERFACE

uses Crt, Dos, Printer, Graph;

const Noir  = 0;           Bleu = 1;             Vert = 2;
      Cyan = 3;            Rouge = 4;            Magenta = 5;
      Brun = 6;            GrisClair = 7;        GrisFonce = 8;
      BleuClair = 9;       VertClair = 10;       CyanClair = 11;
      RougeClair = 12;     MagentaClair = 13;    Jaune = 14;
      Blanc = 15;

var  GraphDriver, GraphMode         : integer;
     XgFen, XdFen, YbFen, YhFen     : real;
     XgClot, XdClot, YbClot, YhClot : integer;
     XP1, YP1, XP2, YP2             : real;
     XOrig, YOrig                   : real;
     XRapport, YRapport             : real;
     Ferme, Ouvert                  : string [6];
     MaxX, MaxY                     : integer;
     NbreCouleurs                   : byte;

type Table = array [1..300] of real;

procedure EcranGraphique;
procedure EcranTexte;
procedure EffaceEcran;
procedure Fenetre (F1, F2, F3, F4 : real);
procedure Cloture (C1, C2, C3, C4 : integer);
procedure PleineCloture;
procedure Couleur (Coul: byte);
procedure Decoupage (X1, Y1, X2, Y2: real);
procedure TraceVers (X, Y : real);
procedure DeplaceEn (X, Y : real);
procedure Bordure (Coul: byte);
procedure TraceBordure;
procedure EffaceBordure;
procedure Axes;
procedure CorrectionAxeX (XOrig,XgFen,UnitX:real;var CorrectX:real);
procedure CorrectionAxeY (YOrig,YbFen,UnitY:real;var CorrectY:real);
procedure Gradue (UnitX, UnitY : real);
procedure Grille (UnitX, UnitY : real);
procedure Polygone (X, Y: table; Lim: integer; Mode: string);
procedure DessinePolygone (X, Y: table; Lim: integer; Mode: string);
procedure EffacePolygone (X, Y: Table; Lim: integer; Mode: string);
procedure TraceCercle (XC, YC, R: real; Pointille: boolean);
procedure Cercle (XC, YC, R:real);
procedure CerclePointille (XC, YC, R: real);
procedure Attente;
procedure Bip;
function Signe (N: real): real;
function Puissance (Base, Expo : real): real;
function Factorielle (X: integer) : real;

IMPLEMENTATION

procedure EcranGraphique;
var ErrorCode : integer;
begin
   GraphDriver := detect;
   InitGraph (GraphDriver, GraphMode, 'c:\bp\bgi');
   ErrorCode := GraphResult;
   if ErrorCode <> grOK then
      begin
         writeln ('Erreur graphique : ',GraphErrorMsg (ErrorCode));
         Halt (1)
      end;
   MaxX := GetMaxX;
   MaxY := GetMaxY;
   NbreCouleurs := GetMaxColor;
   SetColor (Blanc)
end;

procedure EcranTexte;
begin
   CloseGraph
end;

procedure EffaceEcran;
begin
   ClearDevice
end;

procedure Fenetre (F1, F2, F3, F4 : real);
begin
   XgFen := F1;
   XdFen := F2;
   YbFen := F3;
   YhFen := F4
end;

procedure Cloture (C1, C2, C3, C4 : integer);
begin
   XgClot   := C1;
   XdClot   := C2;
   YbClot   := C3;
   YhClot   := C4;
   XRapport := (XdClot - XgClot) / (XdFen - XgFen);
   YRapport := (YhClot - YbClot) / (YhFen - YbFen);
   SetViewport (XgClot, MaxY-YhClot, XdClot, MaxY-YbClot, ClipOn)
end;

procedure PleineCloture;
begin
   Cloture (0, MaxX, 0, MaxY)
end;

procedure Couleur (Coul: byte);
begin
   SetColor (Coul)
end;

procedure Decoupage (X1, Y1, X2, Y2: real);
type Region = (Gauche, Droite, Basse, Haute);
     Code   = set of Region;
var  C, C1, C2 : Code;
     X, Y      : real;
     XX1, YY1  : integer;
     XX2, YY2  : integer;

     procedure CodeBinaire (X, Y: real; var C: Code);
     begin
        C := [];
        if X < XgFen then C := [Gauche]
                     else if X > XdFen then C := [Droite];
        if Y < YbFen then C := C + [Basse]
                     else if Y > YhFen then C := C + [Haute]
     end;

begin
   CodeBinaire (X1, Y1, C1);
   CodeBinaire (X2, Y2, C2);
   while (C1 <> []) or (C2 <> []) do
   begin
      if (C1 * C2) <> [] then Exit;
      if C1 = [] then C := C2 else C := C1;
      if Gauche in C
      then
         begin
            X := XgFen;
            Y := Y1 + (Y2 - Y1) * (XgFen - X1) / (X2 - X1)
         end
      else if Droite in C
           then
              begin
                 X := XdFen;
                 Y := Y1 + (Y2 - Y1) * (XdFen - X1) / (X2 - X1)
              end
           else if Basse in C
                then
                  begin
                     Y := YbFen;
                     X := X1 + (X2 - X1) * (YbFen - Y1) / (Y2 - Y1)
                  end
                else if Haute in C
                     then
                        begin
                           Y := YhFen;
                           X := X1 + (X2-X1) * (YhFen-Y1) / (Y2-Y1)
                        end;
      if C = C1
         then begin
                 X1 := X;
                 Y1 := Y;
                 CodeBinaire (X, Y, C1)
              end
         else begin
                 X2 := X;
                 Y2 := Y;
                 CodeBinaire (X, Y, C2)
              end
   end;
   XX1 := round ((X1 - XgFen) * XRapport);
   YY1 := round ((YhFen - Y1) * YRapport);
   XX2 := round ((X2 - XgFen) * XRapport);
   YY2 := round ((YhFen - Y2) * YRapport);
   MoveTo (XX1, YY1);
   LineTo (XX2, YY2)
end;

procedure TraceVers (X, Y : real);
begin
   XP2 := X;
   YP2 := Y;
   Decoupage (XP1, YP1, XP2, YP2);
   XP1 := XP2;
   YP1 := YP2
end;

procedure DeplaceEn (X, Y : real);
begin
   XP1 := X;
   YP1 := Y;
   TraceVers (X, Y)
end;

procedure Bordure (Coul: byte);
begin
   SetColor (Coul);
   DeplaceEn (Xgfen, YbFen);
   TraceVers (XdFen, YbFen);
   TraceVers (XdFen, YhFen);
   TraceVers (Xgfen, YhFen);
   TraceVers (XgFen, YbFen)
end;

procedure TraceBordure;
begin
   Bordure (Blanc)
end;

procedure EffaceBordure;
begin
   Bordure (Noir)
end;

procedure Axes;
const dX = 5;
      dY = 4;

   procedure PointeDeFlecheSurAxeX;
   begin
      MoveRel (-dX, dY);
      LineRel ( dX,-dY);
      MoveRel (-dX,-dY);
      LineRel ( dX, dY)
   end;

   procedure PointeDeFlecheSurAxeY;
   begin
      MoveRel (-dY, dX);
      LineRel ( dY,-dX);
      MoveRel ( dY, dX);
      LineRel (-dY,-dX)
   end;

begin
   if (XgFen < 0) and (XdFen > 0)
      then XOrig := 0
      else Xorig := XgFen;
   if (YbFen < 0) and (YhFen > 0)
      then YOrig := 0
      else YOrig := YbFen;
   DeplaceEn (XgFen, YOrig);
   TraceVers (XdFen, YOrig);
   PointeDeFlecheSurAxeX;
   DeplaceEn (XOrig, YbFen);
   TraceVers (XOrig, YhFen);
   PointeDeFlecheSurAxeY
end;

procedure CorrectionAxeX (XOrig,XgFen,UnitX:real;var CorrectX:real);
var NbreTirets: real;
begin
  if XOrig = 0
  then begin
         NbreTirets := (XOrig - XgFen) / UnitX;
         CorrectX   := (NbreTirets - int(NbreTirets)) * UnitX
       end
  else if XOrig > 0 then CorrectX:=trunc(XOrig/UnitX+1)*UnitX-XOrig
                    else CorrectX:=abs(XOrig)+trunc(XOrig/UnitX)*UnitX
end;

procedure CorrectionAxeY (YOrig,YbFen,UnitY:real;var CorrectY:real);
var NbreTirets: real;
begin
  if YOrig = 0
  then begin
         NbreTirets := (YOrig - YbFen) / UnitY;
         CorrectY   := (NbreTirets - int(NbreTirets)) * UnitY
       end
  else if YOrig > 0 then CorrectY:=trunc(YOrig/UnitY+1)*UnitY-YOrig
                    else CorrectY:=abs(YOrig)+trunc(YOrig/UnitY)*UnitY
end;

procedure Gradue (UnitX, UnitY : real);
var CorrectX, CorrectY : real;
    X, Y               : real;
    TiretX, TiretY     : byte;
begin
   TiretX := 3;
   TiretY := 4;
   if UnitX > 0 then
      begin
         CorrectionAxeX (XOrig, XgFen, UnitX, CorrectX);
         X := XgFen + CorrectX;
         repeat
            DeplaceEn (X, YOrig);
            TraceVers (X, YOrig);
            MoveRel (0, TiretX);
            LineRel (0, -2 * TiretX);
            X := X + UnitX
         until X > XdFen
      end;
   if UnitY > 0 then
      begin
         CorrectionAxeY (YOrig, YbFen, UnitY, CorrectY);
         Y := YbFen + CorrectY;
         repeat
            DeplaceEn (XOrig, Y);
            TraceVers (XOrig, Y);
            MoveRel (TiretY, 0);
            LineRel (-2 * TiretY, 0);
            Y := Y + UnitY
         until Y > YhFen
      end
end;

procedure Grille (UnitX, UnitY : real);
var  CorrectX, CorrectY, I, J : real;
begin
   CorrectionAxeX (XOrig, XgFen, UnitX, CorrectX);
   CorrectionAxeY (YOrig, YbFen, UnitY, CorrectY);
   I := trunc(XgFen + CorrectX);
   while I <= trunc(XdFen) do
   begin
      J := trunc(YbFen + CorrectY);
      while J <= trunc(YhFen) do
      begin
         DeplaceEn (I, J);
         TraceVers (I, J);
         J := J + UnitY
      end;
      I := I + UnitX
   end
end;

procedure Polygone (X, Y: table; Lim: integer; Mode: string);
var I : integer;
begin
   DeplaceEn (X[1], Y[1]);
   for I := 2 to LIM do TraceVers (X[I], Y[I]);
   if Mode = Ferme then TraceVers (X[1], Y[1])
end;

procedure DessinePolygone (X, Y: table; Lim: integer; Mode: string);
begin
   Couleur (Blanc);
   Polygone (X, Y, Lim, Mode)
end;

procedure EffacePolygone (X, Y: Table; Lim: integer; Mode: string);
begin
   Couleur (Noir);
   Polygone (X, Y, Lim, Mode)
end;

procedure TraceCercle (XC, YC, R: real; Pointille: boolean);
var   S, C, X, Y, Aux : real;
      N               : integer;
begin
   S := sin(Pi / 18);
   C := cos(Pi / 18);
   X := XC + R;
   Y := YC;
   DeplaceEn (X, Y);
   for N := 2 to 37 do
      begin
         Aux := XC + (X - XC) * C - (Y - YC) * S;
         Y   := YC + (X - XC) * S + (Y - YC) * C;
         X   := Aux;
         if Pointille then DeplaceEn (X, Y);
         TraceVers (X, Y)
      end
end;

procedure Cercle (XC, YC, R:real);
begin
   TraceCercle (XC, YC, R, false)
end;

procedure CerclePointille (XC, YC, R: real);
begin
   TraceCercle (XC, YC, R, true)
end;

procedure Attente;
var Ch : char;
begin
   sound(400);
   delay(100);
   nosound;
   repeat until KeyPressed;
   Ch := ReadKey
end;

procedure Bip;
begin
   sound(2000);
   delay(50);
   nosound
end;

function Signe (N: real): real;
begin
   if N >= 0 then Signe := 1 else Signe := -1
end;

function Puissance (Base, Expo : real): real;
begin
  if abs(Base) < 1E-6
    then if Expo = 0.0
           then Puissance := 1.0
           else Puissance := 0.0
    else if Base > 0
           then Puissance := exp (Expo * ln (Base))
           else if abs(Expo) >= 1
                   then if odd(round(Expo))
                          then Puissance := Signe(Base)*exp(Expo*ln(abs(Base)))
                          else Puissance := -Signe(Base)*exp(Expo*ln(abs(Base)))
                   else Puissance := Signe(Base)*exp(Expo*ln(abs(Base)));

end;

function Factorielle (X: integer) : real;
var I : integer;
    F : real;
begin
   if X < 2
      then Factorielle := 1
      else
         begin
            F := 1;
            for I := 2 to X do F := F * I;
            Factorielle := F
         end
end;

END.
