UNIT GRAPH3D;

INTERFACE

uses Crt, Printer, Dos, Graph;

const Blanc     = 15;     {Ndes couleurs pour carte monochrome}
      Noir      = 0;
      Echelle   = 1.35;   { Pour qu'une sphre ait l'air d'une
          sphre.Cette constante doit tre adapte par vous selon
          l'cran utilis.}

Type GenreDeProjection = (perspective, parallele);

Var Rho, Theta, Phi, DE     : real;
    Aux1, Aux2, Aux3, Aux4  : real;
    Aux5, Aux6, Aux7, Aux8  : real;
    Projection              : GenreDeProjection;
    XObs, YObs, ZObs        : real;
    Xproj, YProj            : real;
    XEcran, YEcran          : integer;
    GraphDriver,  GraphMode : integer;
    MaxX, MaxY              : integer;
    Ferme, Ouvert           : string;
    NbreCouleurs            : byte;

procedure EcranGraphique;
procedure EcranTexte;
procedure EffaceEcran;
procedure Couleur (Coul: byte);
procedure Cloture (C1, C2, C3, C4: integer);
procedure InitialiseProjection;
procedure Projette (X, Y, Z: real);
procedure TraceVers (X, Y, Z: real);
procedure DeplaceEn (X, Y, Z: real);
procedure Axes;
procedure AxesPointilles;
procedure Attente;
procedure Bip;

IMPLEMENTATION

procedure EcranGraphique;
var ErrorCode : integer;
begin
   GraphDriver := detect;
   InitGraph (GraphDriver, GraphMode, '');
   ErrorCode := GraphResult;
   if ErrorCode <> grOK then
   begin
      writeln ('Erreur graphique:',GraphErrorMsg (ErrorCode));
      Halt (1)
   end;
   MaxX := GetMaxX;
   MaxY := GetMaxY;
   NbreCouleurs := GetMaxColor;
   SetColor (Blanc)
end;

procedure EcranTexte;
begin
   CloseGraph
end;

procedure EffaceEcran;
begin
   ClearDevice
end;

procedure Couleur (Coul: byte);
begin
   SetColor (Coul)
end;

procedure Cloture (C1, C2, C3, C4: integer);
begin
   SetViewPort (C1, MaxY - C4, C2, MaxY - C3, ClipOn)
end;

procedure InitialiseProjection;
var Th, Ph : real;
begin
   Th   := Pi * Theta / 180;
   Ph   := Pi * Phi / 180;
   Aux1 := sin(Th);
   Aux2 := sin(Ph);
   Aux3 := cos(Th);
   Aux4 := cos(Ph);
   Aux5 := Aux3 * Aux2;
   Aux6 := Aux1 * Aux2;
   Aux7 := Aux3 * Aux4;
   Aux8 := Aux1 * Aux4
end;

procedure Projette (X, Y, Z: real);
begin
   XObs := -X * Aux1 + Y * Aux3;
   YObs := -X * Aux5 - Y * Aux6 + Z * Aux4;
   if Projection = Perspective
   then begin
           ZObs := -X * Aux7 - Y * Aux8 - Z * Aux2 + Rho;
           XProj := DE * XObs / ZObs;
           YProj := DE * YObs / ZObs
        end
   else begin
           XProj := DE * XObs;
           YProj := DE * YObs
        end
end;

procedure TraceVers (X, Y, Z: real);
begin
   Projette (X, Y ,Z);
   XEcran := round(XProj * Echelle + MaxX div 2);
   YEcran := round(MaxY div 2 - YProj);
   LineTo (XEcran, YEcran)
end;

procedure DeplaceEn (X, Y, Z: real);
begin
   Projette (X, Y, Z);
   XEcran := round(XProj * Echelle + MaxX div 2);
   YEcran := round(MaxY div 2 - YProj);
   MoveTo (XEcran, YEcran)
end;

procedure Axes;
begin
   DeplaceEn (0, 0, 0);   TraceVers (10, 0, 0);
   DeplaceEn (0, 0, 0);   TraceVers (0, 10, 0);
   DeplaceEn (0, 0, 0);   TraceVers (0, 0, 10)
end;

procedure AxesPointilles;
begin
   setlinestyle (DottedLn,0,NormWidth);
   DeplaceEn (0, 0, 0);  TraceVers (3, 0, 0);
   MoveRel (5, 5);  OutText ('X');
   DeplaceEn (0, 0, 0);  TraceVers (0, 3, 0);
   MoveRel (5, 5);  OutText ('Y');
   DeplaceEn (0, 0, 0);  TraceVers (0, 0, 3);
   MoveRel (5, 5);  OutText ('Z');
   SetLineStyle (SolidLn, 0, NormWidth)
end;

procedure Attente;
var Ch: char;
begin
   sound(400);   delay(200);   nosound;
   repeat until KeyPressed;
   Ch := ReadKey
end;

procedure Bip;
begin
   sound(2000);
   delay(50);
   nosound
end;

END.
