UNIT PLOT3D;         (* PLOTTER  ROLAND DXY-880A *)

INTERFACE

uses crt, printer, Graph;

Type Table             = array [1..100] of real;
     GenreDeProjection = (perspective, parallele);

Var Rho, Theta, Phi, DE      : real;
    Aux1, Aux2, Aux3, Aux4   : real;
    Aux5, Aux6, Aux7, Aux8   : real;
    XgClot, YbClot           : real;
    XdClot, YhClot           : real;
    Projection               : GenreDeProjection;
    XEcran, YEcran           : integer;
    CorrectionX, CorrectionY : real;

procedure Cloture (C1, C2, C3, C4 : real);
procedure Plume (Numero: byte);
procedure InitialiseProjection;
procedure Projette (X, Y, Z: real);
procedure TraceVers (X, Y, Z: real);
procedure DeplaceEn (X, Y, Z: real);
procedure Axes;
procedure AxesPointilles;
procedure DessinePolygone (X, Y, Z: Table; Lim: integer; Mode: byte);
procedure Attente;
procedure Bip;

IMPLEMENTATION

procedure Cloture (C1, C2, C3, C4 : real);
begin
   XgClot  := C1 * 100;
   XdClot  := C2 * 100;
   YbClot  := C3 * 100;
   YhClot  := C4 * 100;
   CorrectionX := XgClot + (XdClot - XgClot) / 2;
   CorrectionY := YbClot + (YhClot - YbClot) / 2;
   writeln (lst, '^IW', XgClot, ',', YbClot, ',', XdClot, ',', YhClot)
end;

procedure Plume (Numero: byte);
begin
   writeln (lst, 'J', Numero)
end;

procedure InitialiseProjection;
var Th, Ph : real;
begin
   Th   := Pi * Theta / 180;
   Ph   := Pi * Phi / 180;
   Aux1 := sin(Th);
   Aux2 := sin(Ph);
   Aux7 := cos(Th);
   Aux8 := cos(Ph);
   Aux3 := Aux7 * Aux2;
   Aux4 := Aux1 * Aux2;
   Aux5 := Aux7 * Aux8;
   Aux6 := Aux1 * Aux8
end;

procedure Projette (X, Y, Z: real);
var XObs, YObs, ZObs  : real;
begin
   XObs := -X * Aux1 + Y * Aux7;
   YObs := -X * Aux3 - Y * Aux4 + Z * Aux8;
   if Projection = Perspective
       then begin
               ZObs := -X * Aux5 - Y * Aux6 - Z * Aux2 + Rho;
               XEcran := round(DE * XObs / ZObs * 100 + CorrectionX);
               YEcran := round(DE * YObs / ZObs * 100 + CorrectionY)
            end
       else begin
               XEcran := round(DE * XObs * 100 + CorrectionX);
               YEcran := round(DE * YObs * 100 + CorrectionY)
            end
end;

procedure TraceVers (X, Y, Z: real);
begin
    Projette (X, Y ,Z);
    writeln (lst, 'D',XEcran, ',', YEcran)
end;

procedure DeplaceEn (X, Y, Z: real);
begin
    Projette (X, Y, Z);
    writeln (lst, 'M', XEcran, ',', YEcran)
end;

procedure Axes;
begin
   DeplaceEn (0, 0, 0);
   TraceVers (1, 0, 0);
   writeln (lst, 'R',5,',',5); writeln (lst, 'P','X');
   DeplaceEn (0, 0, 0);
   TraceVers (0, 1, 0);
   writeln (lst, 'R',5,',',5); writeln (lst, 'P','Y');
   DeplaceEn (0, 0, 0);
   TraceVers (0, 0, 1);
   writeln (lst, 'R',5,',',5); writeln (lst, 'P','Z');
end;

procedure AxesPointilles;
begin
   setlinestyle (DottedLn,0,NormWidth);
   DeplaceEn (0, 0, 0);
   TraceVers (5, 0, 0);
   DeplaceEn (0, 0, 0);
   TraceVers (0, 5, 0);
   DeplaceEn (0, 0, 0);
   TraceVers (0, 0, 5);
   SetLineStyle (SolidLn, 0, NormWidth);
end;

procedure DessinePolygone (X, Y, Z: Table; Lim: integer; Mode: byte);
var I : integer;
begin
   DeplaceEn (X[1], Y[1], Z[1]);
   for I := 2 to LIM do TraceVers (X[I], Y[I], Z[I]);
   if Mode=1 then TraceVers (X[1], Y[1], Z[1])
end;

procedure Attente;
begin
   sound(400);
   delay(200);
   nosound;
   readln
end;

procedure Bip;
begin
   sound(2000);
   delay(50);
   nosound
end;

END.
