program SURFICO;         { Surfaces caches d'un polydre convexe: icosadre }

Uses Crt, Graph, GRAPH3D;

const MaxSommet = 20;
      MaxFaces  = 20;
      MaxAretes = 30;

var St         : array [1..MaxSommet, 1..3] of real;
    Fc         : array [1..MaxFaces, 0..MaxAretes] of integer;
    O1, O2, O3 : real;
    NF         : integer;
    Tiret      : boolean;
    IncEcran   : real;
    IncRho     : real;
    IncAng     : real;

procedure Initialisation;
begin
   Projection := Perspective;
   Rho   := 5;           IncAng   := 5;
   Theta := 0;           IncRho   := 1;
   Phi   := 0;           IncEcran := 20;
   DE    := 300;         Tiret    := true
end;

procedure LectureSommets;
const C1 = 0.8944271;
      C2 = 0.4472137;
      C3 = 0.2763932;
      C4 = 0.8506507;
      C5 = 0.7236067;
      C6 = 0.5257311;
begin
   St[1,1]  := C1;      St[1,2]  := 0;       St[1,3]  := C2;
   St[2,1]  := C3;      St[2,2]  := C4;      St[2,3]  := C2;
   St[3,1]  :=-C5;      St[3,2]  := C6;      St[3,3]  := C2;
   St[4,1]  :=-C5;      St[4,2]  :=-C6;      St[4,3]  := C2;
   St[5,1]  := C3;      St[5,2]  :=-C4;      St[5,3]  := C2;
   St[6,1]  := 0;       St[6,2]  := 0;       St[6,3]  := 1;
   St[7,1]  := 0;       St[7,2]  := 0;       St[7,3]  := -1;
   St[8,1]  := C5;      St[8,2]  := C6;      St[8,3]  :=-C2;
   St[9,1]  :=-C3;      St[9,2]  := C4;      St[9,3]  :=-C2;
   St[10,1] :=-C1;      St[10,2] := 0;       St[10,3] :=-C2;
   St[11,1] :=-C3;      St[11,2] :=-C4;      St[11,3] :=-C2;
   St[12,1] := C5;      St[12,2] :=-C6;      St[12,3] :=-C2
end;

procedure LectureFaces;
var I : integer;
begin
   NF:= 20;
   for I := 1 to NF do Fc[I,0] := 3;
   Fc[1,1] :=1;      Fc[1,2] :=2;      Fc[1,3] :=6;
   Fc[2,1] :=2;      Fc[2,2] :=3;      Fc[2,3] :=6;
   Fc[3,1] :=3;      Fc[3,2] :=4;      Fc[3,3] :=6;
   Fc[4,1] :=4;      Fc[4,2] :=5;      Fc[4,3] :=6;
   Fc[5,1] :=5;      Fc[5,2] :=1;      Fc[5,3] :=6;
   Fc[6,1] :=12;     Fc[6,2] :=1;      Fc[6,3] :=5;
   Fc[7,1] :=12;     Fc[7,2] :=8;      Fc[7,3] :=1;
   Fc[8,1] :=8;      Fc[8,2] :=2;      Fc[8,3] :=1;
   Fc[9,1] :=9;      Fc[9,2] :=2;      Fc[9,3] :=8;
   Fc[10,1]:=9;      Fc[10,2]:=3;      Fc[10,3]:=2;
   Fc[11,1]:=9;      Fc[11,2]:=10;     Fc[11,3]:=3;
   Fc[12,1]:=10;     Fc[12,2]:=4;      Fc[12,3]:=3;
   Fc[13,1]:=10;     Fc[13,2]:=11;     Fc[13,3]:=4;
   Fc[14,1]:=11;     Fc[14,2]:=5;      Fc[14,3]:=4;
   Fc[15,1]:=11;     Fc[15,2]:=12;     Fc[15,3]:=5;
   Fc[16,1]:=7;      Fc[16,2]:=8;      Fc[16,3]:=12;
   Fc[17,1]:=7;      Fc[17,2]:=9;      Fc[17,3]:=8;
   Fc[18,1]:=7;      Fc[18,2]:=10;     Fc[18,3]:=9;
   Fc[19,1]:=7;      Fc[19,2]:=11;     Fc[19,3]:=10;
   Fc[20,1]:=7;      Fc[20,2]:=12;     Fc[20,3]:=11
end;

procedure VecteurVision (St1, St2, St3: integer; var V1, V2, V3: real);
begin
   V1 := O1 - St[St1,1];
   V2 := O2 - St[St1,2];
   V3 := O3 - St[St1,3]
end;

procedure VecteurNormal (St1, St2, St3: integer; var N1, N2, N3: real);
var P1, P2, P3, Q1, Q2, Q3 :real;
begin
   P1 := St[St2,1] - St[St1,1];
   P2 := St[St2,2] - St[St1,2];
   P3 := St[St2,3] - St[St1,3];
   Q1 := St[St3,1] - St[St1,1];
   Q2 := St[St3,2] - St[St1,2];
   Q3 := St[St3,3] - St[St1,3];
   N1 := P2 * Q3 - Q2 * P3;
   N2 := P3 * Q1 - Q3 * P1;
   N3 := P1 * Q2 - Q1 * P2
end;

function ProduitScalaire (V1, V2, V3, N1, N2, N3: real): real;
begin
   ProduitScalaire := V1 * N1 + V2 * N2 + V3 * N3
end;

procedure DessineObjet;
var F, St1, St2, St3, NS, No :integer;
    V1 , V2, V3, N1, N2, N3  : real;
    X, Y, Z, X0, Y0, Z0      : real;

   procedure DessineFace;
   var S: integer;
   begin
      NS := Fc[F, 0];
      for S := 1 to NS do
      begin
         No := Fc[F,S];
         X  := St[No,1];
         Y  := St[No,2];
         Z  := St[No,3];
         if S = 1 then begin
                          DeplaceEn (X,Y,Z);
                          X0 := X;  Y0 := Y;  Z0 := Z
                       end
                  else TraceVers (X,Y,Z)
      end;
      TraceVers (X0, Y0, Z0)
   end;

begin
   for F := 1 to NF do
   begin
      St1 := Fc[F,1];    St2 := Fc[F,2];     St3 := Fc[F,3];
      VecteurVision (St1, St2, St3, V1, V2, V3);
      VecteurNormal (St1, St2, St3, N1, N2, N3);
      if ProduitScalaire (V1, V2, V3, N1, N2, N3) >= 0
      then begin
              SetLineStyle (SolidLn, 0, NormWidth);
              DessineFace
           end
      else if Tiret then begin
                            SetLineStyle (DottedLn, 0, NormWidth);
                            DessineFace
                         end
   end
end;

procedure Dessine;
begin
   InitialiseProjection;
   O1 := Rho * Aux7;
   O2 := Rho * Aux8;
   O3 := Rho * Aux2;
   DessineObjet
end;

procedure Affichage;
var S1, S2, S3, S4, S5 : string;
begin
   Cloture (0, MaxX, 0, 15);
   ClearViewPort;
   SetTextStyle (SmallFont, HorizDir, 5);
   str (Theta:3:1, S2);
   str (Phi:3:1, S3);
   str (DE:4:1, S4);
   if projection = Perspective
   then begin
           str (Rho:3:1, S1);
           OutTextXY (90, 1,'Projection Perspective: Rho='+S1+
                            '   Theta= '+S2+'   Phi='+S3+'  Ecran='+S4)
        end
   else OutTextXY (90, 1,'Projection Parallele: Rho=infini'+
                         '   Theta= '+S2+'   Phi='+S3+'  Ecran='+S4);
   Cloture (0, MaxX, 0, MaxY)
end;

procedure Commandes;
const RhoPara = 1E20;
var RhoPersp, DEPersp, DEPara: real;
    Ch : char;
begin
   Initialisation;
   DEPara := 50;
   Dessine;
   Affichage;
   repeat
      Ch := UpCase(ReadKey);
      if Ch = #0 then Ch := UpCAse(ReadKey);
      if ord(Ch) in [72, 80, 75, 77, 69, 65, 43, 45, 84, 67]
      then
         begin
            ClearDevice;
            case ord(Ch) of
               72 : Phi   := Phi + IncAng;        { flche haut }
               80 : Phi   := Phi - IncAng;        { flche bas  }
               75 : Theta := Theta - IncAng;      { flche gauche }
               77 : Theta := Theta + IncAng;      { flche droite }
               69 : Rho   := Rho + IncRho;        { E pour s'loigner }
               65 : Rho   := Rho - IncRho;        { A pour s'approcher }
               43 : DE    := DE + IncEcran;       { + pour augmenter distance cran }
               45 : DE    := DE - IncEcran;       { - pour diminuer distance cran }
               84 : Tiret := not(Tiret);          { T pour voir ou non les tirets }
               67 : If Projection = Perspective   { C pour changer de projection }
                    then begin
                            RhoPersp := Rho;
                            DEPersp  := DE;
                            Projection := Parallele;
                            Rho := RhoPara;
                            DE  := DEPara
                         end
                    else begin
                            DEPAra  := DE;
                            Projection := Perspective;
                            Rho := RhoPersp;
                            DE  := DEPersp
                         end
            end;
            Dessine;
            Affichage
         end
   until UpCase(Ch) = 'F';
   EcranTexte
end;

BEGIN
   EcranGraphique;
   Cloture (0, MaxX, 0, MaxY);
   LectureSommets;
   LectureFaces;
   Initialisation;
   Commandes;
END.
